"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.defaultOptions = exports.TRANSITION_RANDOM = exports.TRANSITION_PROGRESSIVE = exports.TRANSITION_CONSTANT = void 0;

var _measureText = _interopRequireDefault(require("./measureText"));

var _rejectUndefined = _interopRequireDefault(require("./rejectUndefined"));

var _getCharYPos = _interopRequireDefault(require("./getCharYPos"));

var _defaultConstants = require("../defaultConstants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const TRANSITION_CONSTANT = 'constant';
exports.TRANSITION_CONSTANT = TRANSITION_CONSTANT;
const TRANSITION_PROGRESSIVE = 'progressive';
exports.TRANSITION_PROGRESSIVE = TRANSITION_PROGRESSIVE;
const TRANSITION_RANDOM = 'random';
exports.TRANSITION_RANDOM = TRANSITION_RANDOM;
const defaultOptions = {
  width: _defaultConstants.DEFAULT_WIDTH,
  height: _defaultConstants.DEFAULT_HEIGHT,
  fontSize: _defaultConstants.DEFAULT_FONT_SIZE,
  lineHeight: _defaultConstants.DEFAULT_LINE_HEIGHT,
  fontFamily: _defaultConstants.DEFAULT_FONT_FAMILY,
  transitionStyle: _defaultConstants.DEFAULT_TRANSITION_STYLE,
  transitionSpeedMultiplier: _defaultConstants.DEFAULT_TRANSITION_SPEED_MULTIPLIER,
  charCenters: undefined
};
exports.defaultOptions = defaultOptions;

const generateTransition = (type, speedMultiplier, index = 0) => {
  switch (type) {
    case TRANSITION_CONSTANT:
      return `all ${1 * speedMultiplier}s`;

    case TRANSITION_PROGRESSIVE:
      return `all ${(1 + 0.15 * index) * speedMultiplier}s`;

    case TRANSITION_RANDOM:
      return `all ${(1 + Math.random()) * speedMultiplier}s`;

    default:
      return 'all 1s';
  }
};
/**
 * Generates the styles for the wrapper and each of the text characters for the animation to happen.
 *
 * @param {string} text text to be rendered. It is used to generate a style for each character.
 * @param {object} [options={}] object with options
 * @param {string} [options.fontSize='20px'] fontSize of the text when the user hovers it.
 * @param {number} [options.lineHeight=1.3] lineHeight of the text when the user hovers it.
 * @param {string} [options.fontFamily='Georgia'] fontFamily of the text.
 * @param {string} [options.transitionStyle='constant'] style of the transition animation. Values: 'constant' | 'progressive' | 'random'.
 * @param {string} [options.transitionSpeedMultiplier=1] speed multiplier for the transition. Default transitions take 1s. The multiplier can increase and decrease that.
 * @param {Array<{ x: number, y: number }>} [options.charCenters=undefined] position of the characters before the user hovers them.
 * @returns {{[styleName]: styles}} returns a object with the JSS styles for the wrapper and each character. This object needs to be consumed
 * by the `withStyles` HoC from Material-UI to generate the CSS. The `injectStyles` HoC from JSS can also be used.
 */


const createStyles = (text, options = defaultOptions) => () => {
  const _defaultOptions$rejec = _objectSpread({}, defaultOptions, (0, _rejectUndefined.default)(options)),
        width = _defaultOptions$rejec.width,
        height = _defaultOptions$rejec.height,
        fontSize = _defaultOptions$rejec.fontSize,
        lineHeight = _defaultOptions$rejec.lineHeight,
        fontFamily = _defaultOptions$rejec.fontFamily,
        transitionStyle = _defaultOptions$rejec.transitionStyle,
        transitionSpeedMultiplier = _defaultOptions$rejec.transitionSpeedMultiplier,
        charCenters = _defaultOptions$rejec.charCenters;

  const textAsArray = text.split('');
  const textWidth = (0, _measureText.default)(text, {
    fontFamily,
    fontSize,
    lineHeight
  }).width.value;
  const textCharWidths = textAsArray.map(char => (0, _measureText.default)(char, {
    fontFamily,
    fontSize,
    lineHeight
  }).width.value);
  const charsHoverStyle = textAsArray.reduce((acc, char, index) => {
    acc[`&:hover $char-${index}`] = _objectSpread({
      top: '50%',
      transform: `rotate(0deg)`,
      fontSize
    }, (0, _getCharYPos.default)(index, textWidth, textCharWidths));
    return acc;
  }, {});
  const charsDefaultStyle = charCenters ? textAsArray.reduce((acc, char, index) => {
    acc[`char-${index}`] = {
      position: 'absolute',
      transition: generateTransition(transitionStyle, transitionSpeedMultiplier, index),
      left: `${charCenters[index].x / width * 100}%`,
      top: `${charCenters[index].y / height * 100}%`,
      fontSize: '50px',
      transform: `rotate(${Math.random() * 90 - Math.random() * 90}deg)`
    };
    return acc;
  }, {}) : textAsArray.reduce((acc, char, index) => {
    acc[`char-${index}`] = _objectSpread({
      position: 'absolute',
      transition: `all 1s`,
      top: '50%',
      transform: `rotate(0deg)`,
      fontSize
    }, (0, _getCharYPos.default)(index, textWidth, textCharWidths));
    return acc;
  }, {});
  return _objectSpread({
    wrapper: _objectSpread({
      width: '100%',
      height: '100%',
      position: 'relative'
    }, charsHoverStyle)
  }, charsDefaultStyle);
};

var _default = createStyles;
exports.default = _default;