"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.defaultOptions = void 0;

var _unitsCss = _interopRequireDefault(require("units-css"));

var _rejectUndefined = _interopRequireDefault(require("./rejectUndefined"));

var _defaultConstants = require("../defaultConstants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const defaultOptions = {
  fontFamily: _defaultConstants.DEFAULT_FONT_FAMILY,
  fontSize: _defaultConstants.DEFAULT_FONT_SIZE,
  lineHeight: _defaultConstants.DEFAULT_LINE_HEIGHT,
  fontWeight: _defaultConstants.DEFAULT_FONT_WEIGHT,
  fontStyle: _defaultConstants.DEFAULT_FONT_STYLE,
  canvas: _defaultConstants.DEFAULT_CANVAS
};
exports.defaultOptions = defaultOptions;

const measureHeight = (size, lineHeight) => {
  // If the line-height is unitless,
  // multiply it by the font size.
  if (!lineHeight.unit) {
    return _unitsCss.default.parse(`${size.value * lineHeight.value}${size.unit}`);
  } // units-css requires the user to provide
  // DOM nodes for these units. We don't want
  // to pollute our API with that for the time being.


  const unitBlacklist = ['%', 'ch', 'cm', 'em', 'ex'];

  if (unitBlacklist.indexOf(lineHeight.unit) !== -1) {
    // eslint-disable-line no-magic-numbers
    throw new Error(`We do not currently support the unit ${lineHeight.unit}
      from the provided line-height ${lineHeight.value}.
      Unsupported units include ${unitBlacklist.join(', ')}.`);
  } // Otherwise, the height is equivalent
  // to the provided line height.
  // Non-px units need conversion.


  if (lineHeight.unit === 'px') {
    return lineHeight;
  }

  return _unitsCss.default.parse(_unitsCss.default.convert(lineHeight, 'px'));
};
/**
 * Measures the widths and the height of a text
 *
 * @param {string | Array<string>} text text to be measured or array with multiple lines of text.
 * @param {object} [options={}] object with options
 * @param {string} [options.fontSize='20px'] fontSize of the text.
 * @param {number} [options.lineHeight=1.3] lineHeight of the text.
 * @param {string} [options.fontFamily='Georgia'] fontFamily of the text.
 * @param {string} [options.fontWeight=400] fontWeight of the text.
 * @param {string} [options.fontStyle='normal'] fontStyle of the text.
 * @param {string} [options.canvas=created by default] canvas used to render the text to measure. If none provided, one us created and used.
 * @returns {{ text: string, width: { value: number, units: string }, height: string }} returns an object with the width, height and the text or longest text in case
 * of multiline.
 */


const measureText = (text, options = defaultOptions) => {
  const _defaultOptions$rejec = _objectSpread({}, defaultOptions, (0, _rejectUndefined.default)(options)),
        fontFamily = _defaultOptions$rejec.fontFamily,
        fontSize = _defaultOptions$rejec.fontSize,
        lineHeight = _defaultOptions$rejec.lineHeight,
        fontWeight = _defaultOptions$rejec.fontWeight,
        fontStyle = _defaultOptions$rejec.fontStyle,
        canvas = _defaultOptions$rejec.canvas;

  const ctx = canvas.getContext('2d');
  ctx.font = `${fontWeight} ${fontStyle} ${fontSize} ${fontFamily}`;

  const measure = line => {
    return {
      text: line,
      width: _unitsCss.default.parse(`${ctx.measureText(line).width}px`),
      height: measureHeight(_unitsCss.default.parse(fontSize), _unitsCss.default.parse(lineHeight))
    };
  }; // If multiline, measure the bounds
  // of all of the lines combined


  if (Array.isArray(text)) {
    return text.map(measure).reduce((prev, curr) => {
      const width = curr.width.value > prev.width.value ? curr.width : prev.width;

      const height = _unitsCss.default.parse(`${prev.height.value + curr.height.value}${curr.height.unit}`);

      const longest = curr.text.length > prev.text.length ? curr.text : prev.text;
      return {
        width,
        height,
        text: longest
      };
    });
  }

  return measure(text);
};

var _default = measureText;
exports.default = _default;